uniform vec3 BaseColor;
uniform float MixRatio;

uniform samplerCube EnvMap;

uniform bool DoLightMapping;
uniform sampler2D LightMap;

uniform bool DoShadowMapping;
uniform sampler2DShadow ShadowMap;

varying vec3 ReflectDir;

varying vec3 Normal;
varying vec3 LightDir;
varying vec3 HalfVector;

varying float LightDistance;

varying vec4 LightColor;

varying vec4 ShadowCoord;

varying float Fog;

void main()
{
	vec4 originalColor = vec4(textureCube(EnvMap, ReflectDir));
	
	vec4 color = vec4(mix(originalColor.rgb, BaseColor, MixRatio), 1.);
	
	vec3 normal = normalize(Normal);
	float nDotL = max(dot(normal, normalize(LightDir)), 0.);
	
	// lightmap coordinate
	if(DoLightMapping)
	{
		vec4 lightMapColor = texture2DProj(LightMap, gl_TexCoord[1]);
		color = mix(color, lightMapColor, lightMapColor.a * 0.9);
	}
	if(DoShadowMapping)
	{
		float x = ShadowCoord.x;
		float y = ShadowCoord.y;
		float z = ShadowCoord.z;
		if((x >= 0.) && (x <= 1.) && (y >= 0.) && (y <= 1.))
			color *= shadow2D(ShadowMap, ShadowCoord.xyz).x != 1.0 ? 0.5 : 1.0;
	}
	
	gl_FragColor = vec4(vec3(color) * Fog, 1.);
}
