varying vec3 ReflectDir;

varying vec3 Normal;
varying vec3 LightDir;
varying vec3 HalfVector;

varying float LightDistance;

uniform float Phase;
uniform vec3 LightPos;

uniform vec3 LightPositions[30];

varying vec4 LightColor;

vec4 pointLight(vec3 objectPos, vec3 objectNormal, vec3 lightPos);
float fogFactor(float density);

uniform vec3 Displacement;

uniform float FogDensity;
varying float Fog;

// cube map
uniform mat4 CubeMapMatrix;

// light map
const mat4 ProjTexMatrix = mat4(
		0.5, 0  , 0  , 0  ,
		0  , 0.5, 0  , 0  ,
		0  , 0  , 0.5, 0  ,
		0.5, 0.5, 0.5, 1  ); // this matrix is also used for shadow map
		
// shadow map
uniform mat4 LightLookAtMatrix;
uniform mat4 EyeLookAtMatrix;
varying vec4 ShadowCoord;

const float Frequency = 4.;//6.;
const float Amplitude = 0.01;//0.01;//0.001;

const float D  = 0.010; // represents dx or dz
const float Dh = 0.005; // = D/2.

#define YDisplacement(wavePos) sin((wavePos) * Frequency) * Amplitude + \
                               wavePos * 0.0000000000001
#define Slope(wavePos) sin(((wavePos) + Dh) * Frequency) * Amplitude -\
                       sin(((wavePos) - Dh) * Frequency) * Amplitude

void main()
{
	float waveX = gl_Vertex.x + Displacement.x + Phase;
	float waveZ = gl_Vertex.z + Displacement.z + Phase;
	
	gl_Vertex.y += YDisplacement(waveX) + YDisplacement(waveZ);
	float dyx = Slope(waveX);
	float dyz = Slope(waveZ);
	Normal = normalize(gl_NormalMatrix *
	                        (vec3(-dyx, D, 0.) + vec3(0., D, -dyz)));
	
	//gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;
	mat4 mvpMatrix = gl_ProjectionMatrix * EyeLookAtMatrix * gl_ModelViewMatrix;
	gl_Position = mvpMatrix * gl_Vertex;
	
	vec4 pos = CubeMapMatrix * gl_Vertex;
	vec3 eyeDir = pos.xyz;
	
	ReflectDir = reflect(eyeDir, Normal);
	
	// lightmap coordinate
	vec4 xzVertex = gl_Vertex;
	xzVertex.y = 0.;
	vec4 tc = ProjTexMatrix * mvpMatrix * xzVertex;
	gl_TexCoord[1] = tc;
	
	// shadowmap coordinate
	vec4 view = gl_ModelViewMatrix * gl_Vertex;
	view = LightLookAtMatrix * view;
	//view.z += 0.02;
	view.x *= 0.2; // do "projection" of x
	view.y *= 0.3; // do "projection" of y
	view.z *= -0.1; // do "projection" of z
	ShadowCoord = ProjTexMatrix * view;
	
	Fog = fogFactor(FogDensity);
	
}
