varying vec4 Coord;
varying vec3 Pos;
varying vec3 Normal;

uniform bool UseShadowMap;
uniform sampler2DShadow ShadowMap;
varying vec4 ShadowCoord;

varying float Fog;

// grow rings
uniform float Vr; // ring grow variation [1.]
uniform float Fr; // ring frequency [100.]

// deform
uniform vec2 Axy; // x and y deform amplitude [0.3, 0.4]
uniform vec2 Fxy; // x and y deform frequencies [30., 25.]
uniform vec2 Pzz; // z violent and moderate deform displacement [0., 0.]
uniform vec2 Fzz; // z violent and moderate deform frequencies [5., 17.]

// color
uniform vec3 ConstantColor; // constant color [0.2, 0.1, 0.]
uniform vec3 DiffuseColor; // diffuse color [0.7, 0.2, 0.]
uniform vec2 Arg; // red and green modulation amplitude [0.1, 0.03]
uniform vec2 Frg; // red and green modulation frequency [7., 19.]

float clip(float f)
{
	if(f > 1. ) return 1.;
	if(f < 0.2) return 0.2;
	return f;
}

vec4 pointLight(vec3 objectPos, vec3 objectNormal, vec3 lightPos);

void main()
{
	float x = Coord.x;
	float y = Coord.y;
	float z = Coord.z;
	float l = length(Coord.xy);
	float angle = pow(l, 0.5*Vr)*Fr +
			sin(x*Fxy.x)*Axy.x + sin(y*Fxy.y)*Axy.y +
			sin((z+Pzz.x)*Fzz.x) + sin((z+Pzz.y)*Fzz.y)*0.5;
	float f = (sin(angle) + 1.) / 4. + 0.4;
	f = clip(f * pow(l*3., 0.4));
	float fR = clip(f + sin(z*Frg.r)*Arg.r);
	float fG = clip(f + sin(z*Frg.g)*Arg.g);
	
	vec3 c = vec3(gl_Color);
	
	if(UseShadowMap)
	{
		c += pointLight(Pos, Normal, gl_LightSource[0].position.xyz);
		
		float x, y, z;
		x = ShadowCoord.x;
		y = ShadowCoord.y;
		z = ShadowCoord.z;
		if((x > 0.) && (x < 1.) && (y > 0.) && (y < 1.))
		{
			ShadowCoord.xyz /= ShadowCoord.w;
			ShadowCoord.z = step(0., ShadowCoord.z) * step(ShadowCoord.z, 1.);
			x = ShadowCoord.x;
			y = ShadowCoord.y;
			z = ShadowCoord.z;
			if((x > 0.) && (x < 1.) && (y > 0.) && (y < 1.))
			{
				c *= shadow2D(ShadowMap, ShadowCoord.xyz).x;
			}
		}
	}
	vec3 cc = ConstantColor;
	vec3 dc = DiffuseColor;
	gl_FragColor = vec4(Fog*fR * (cc.r+c.r*dc.r), Fog*fG * (cc.g+c.g*dc.g), 0.,
			1.);
	
}
