varying vec4 Coord;
varying vec3 Pos;
varying vec3 Normal;

uniform bool UseSingleLightSource;
uniform vec3 LightPositions[30];

// shadow map
uniform bool UseShadowMap;
const mat4 ProjTexMatrix = mat4(
		0.5, 0  , 0  , 0  ,
		0  , 0.5, 0  , 0  ,
		0  , 0  , 0.5, 0  ,
		0.5, 0.5, 0.5, 1  );
uniform mat4 LightLookAtMatrix;
uniform mat4 EyeLookAtMatrix;
varying vec4 ShadowCoord;

uniform float FogDensity;
uniform float FogBegin;
varying float Fog;

vec4 pointLight(vec3 objectPos, vec3 objectNormal, vec3 lightPos);
float fogFactor(float density);
float fogFactor(float density, float begin);

void main()
{
	
	vec4 color = vec4(0., 0., 0., 1.);
	if(UseSingleLightSource)
	{
		if(UseShadowMap)
		{
			Normal = normalize(mat3(EyeLookAtMatrix) * gl_NormalMatrix *
					gl_Normal);
			Pos = vec3(EyeLookAtMatrix * gl_ModelViewMatrix * gl_Vertex);
		}
		else
		{
			Normal = normalize(gl_NormalMatrix * gl_Normal);
			vec3 pos = vec3(gl_ModelViewMatrix * gl_Vertex);
			vec3 lightDir = normalize(vec3(gl_LightSource[0].position));
			float nDotL = max(dot(Normal, lightDir), 0.);
			color += gl_FrontMaterial.diffuse * gl_LightSource[0].diffuse *
					nDotL;
		}
	}
	else
	{
		Normal = normalize(gl_NormalMatrix * gl_Normal);
		vec3 pos = vec3(gl_ModelViewMatrix * gl_Vertex);
		for(int i = 0; i < 30; i++)
		{
			vec3 lpos = LightPositions[i];
			//lpos.y = -lpos.y;
			color += pointLight(pos, Normal, lpos);
		}
	}
	gl_FrontColor = color;
	
	Coord = gl_Vertex;
	
	if(UseShadowMap)
	{
		mat4 mvpMatrix = gl_ProjectionMatrix * EyeLookAtMatrix *
				gl_ModelViewMatrix;
		gl_Position = mvpMatrix * gl_Vertex;
	
		// shadowmap coordinate
		vec4 view = LightLookAtMatrix * gl_ModelViewMatrix * gl_Vertex;
		//view.z += 0.02;
		view = gl_ProjectionMatrix * view;
		view.x *=  0.2;
		view.y *=  0.5;
		view.z *=  0.1;
		ShadowCoord = ProjTexMatrix * view;
	}
	else
	{
		gl_Position = ftransform();
	}
	
	if(FogBegin != 0.)
		Fog = fogFactor(FogDensity, FogBegin);
	else
		Fog = fogFactor(FogDensity);
	
}
